//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern double Lots               = 0.01;
extern double MaximumRisk        = 0;
extern double DecreaseFactor     = 0;
extern double MovingPeriod       = 1;
extern double MovingShift        = 0;
extern string  Tr_Stop="TrailingStop";
extern int     Stoploss             = 0,     //,  0   
               Takeprofit           = 0;      //,  0   
extern int     TrailingStop         = 0;       // 250  ,  0   
extern int     StepTrall            = 0;       // 250   -      StepTrall
extern int     NoLoss               = 0,       // 100      -  ,  0     
               MinProfitNoLoss      = 0;       // 50     
int  STOPLEVEL,TimeBar; // 
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   {
      ObjectDelete("SLb"); //
   ObjectDelete("SLs"); //
   return(0);
  }  
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,01);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.01) lot=0.01;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//---- sell conditions
   if(Open[2]>ma && Close[2]>ma)  
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
//---- buy conditions
   if(Open[2]<ma && Close[2]<ma)  
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[4]>ma && Close[4]>ma) OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(Open[4]<ma && Close[4]<ma) OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<10 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//----
 // =================  =====================================
{
      STOPLEVEL=MarketInfo(Symbol(),MODE_STOPLEVEL);
      double OSL,OTP,OOP,StLo,SL,TP;
      double Profit,ProfitS,ProfitB,LB,LS,NLb,NLs,price_b,price_s,OL,sl;
      int b,s,OT,OMN;
      for (int ii=OrdersTotal()-1; ii>=0; ii--)
      {                                               
         if (OrderSelect(ii,SELECT_BY_POS,MODE_TRADES))
         {
            OMN = OrderMagicNumber();
            if (OrderSymbol() == Symbol())
            {
               OOP = NormalizeDouble(OrderOpenPrice(),Digits);
               OT = OrderType();
               OL = OrderLots();
               if (OT==OP_BUY)
               {
                  price_b = price_b+OOP*OL;
                  b++; LB+= OL;
                  ProfitB+=OrderProfit()+OrderSwap()+OrderCommission();
               }
               if (OT==OP_SELL)
               {
                  price_s = price_s+OOP*OL;
                  s++;LS+= OL;
                  ProfitS+=OrderProfit()+OrderSwap()+OrderCommission();
               }
            }
         }
      }
      ObjectDelete("SLb");
      ObjectDelete("SLs");
      if (b>0) 
      {
         NLb = NormalizeDouble(price_b/LB,Digits);
         ObjectCreate("SLb",OBJ_ARROW,0,Time[0],NLb,0,0,0,0);                     
         ObjectSet   ("SLb",OBJPROP_ARROWCODE,6);
         ObjectSet   ("SLb",OBJPROP_COLOR, LightSkyBlue);
      }
      if (s>0) 
      {
         NLs = NormalizeDouble(price_s/LS,Digits);
         ObjectCreate("SLs",OBJ_ARROW,0,Time[0],NLs,0,0,0,0);                     
         ObjectSet   ("SLs",OBJPROP_ARROWCODE,6);
         ObjectSet   ("SLs",OBJPROP_COLOR, HotPink);
      }
      int OTicket;
      for (ii=0; ii<OrdersTotal(); ii++)
      {    
         if (OrderSelect(ii,SELECT_BY_POS,MODE_TRADES))
         {
            if (OrderSymbol()==Symbol())
            { 
               OT = OrderType(); 
               OSL = NormalizeDouble(OrderStopLoss(),Digits);
               OTP = NormalizeDouble(OrderTakeProfit(),Digits);
               OOP = NormalizeDouble(OrderOpenPrice(),Digits);
               SL=OSL;TP=OTP;
               if (OT==OP_BUY)             
               {  
                  b++;
                  if (OSL==0 && Stoploss>=STOPLEVEL && Stoploss!=0)
                  {
                      SL = NormalizeDouble(Bid - Stoploss   * Point,Digits);
                  } 
                  else SL=OSL;
                  if (OTP==0 && Takeprofit>=STOPLEVEL && Takeprofit!=0)
                  {
                      TP = NormalizeDouble(Ask + Takeprofit * Point,Digits);
                  } 
                  else TP=OTP;
                  if (NoLoss>=STOPLEVEL && OSL<NLb && NoLoss!=0)
                  {
                     if (OOP<=NLb && NLb!=0 && NLb <= NormalizeDouble(Bid-NoLoss*Point,Digits)) 
                        SL = NormalizeDouble(NLb+MinProfitNoLoss*Point,Digits);
                  }
                  if (TrailingStop>=STOPLEVEL && TrailingStop!=0)
                  {
                     StLo = NormalizeDouble(Bid - TrailingStop*Point,Digits); 
                     if (StLo>=NLb && NLb!=0) if (StLo > OSL) SL = StLo;
                  }
                  if (SL != OSL || TP != OTP)
                  {  
                     OTicket=OrderTicket();
                     if (!OrderModify(OTicket,OOP,SL,TP,0,Gold)) Print("Error OrderModify ",GetLastError());
                  }
               }                                         
               if (OT==OP_SELL)        
               {
                  s++;
                  if (OSL==0 && Stoploss>=STOPLEVEL && Stoploss!=0)
                  {
                     SL = NormalizeDouble(Ask + Stoploss   * Point,Digits);
                  }
                  else SL=OSL;
                  if (OTP==0 && Takeprofit>=STOPLEVEL && Takeprofit!=0)
                  {
                      TP = NormalizeDouble(Bid - Takeprofit * Point,Digits);
                  }
                  else TP=OTP;
                  if (NoLoss>=STOPLEVEL && (OSL>NLs || OSL==0) && NoLoss!=0)
                  {
                     if (OOP>=NLs && NLs!=0 && NLs >= NormalizeDouble(Ask+NoLoss*Point,Digits)) 
                        SL = NormalizeDouble(NLs-MinProfitNoLoss*Point,Digits);
                  }
                  if (TrailingStop>=STOPLEVEL && TrailingStop!=0)
                  {
                     StLo = NormalizeDouble(Ask + TrailingStop*Point,Digits); 
                     if (StLo<=NLs && NLs!=0) if (StLo < OSL || OSL==0) SL = StLo;
                  }
                  if ((SL != OSL || OSL==0) || TP != OTP)
                  {  
                     OTicket=OrderTicket();
                     if (!OrderModify(OTicket,OOP,SL,TP,0,Gold)) Print("Error OrderModify ",GetLastError());
                  }
               } 
            }
         }
      }
   }    
//=================  END =====================================
  }
//+------------------------------------------------------------------+